@@ -1,4 +1,4 @@
-# Copyright 2015, 2016 IBM Corp.
+# Copyright 2015, 2018 IBM Corp.
 #
 # All Rights Reserved.
 #
@@ -19,9 +19,11 @@ from nova.api.metadata import base as instance_metadata
 from nova.network import model as network_model
 from nova.virt import configdrive
 import os
+import retrying
 from taskflow import task
 
 from oslo_log import log as logging
+from oslo_utils import excutils
 
 from pypowervm import const as pvm_const
 from pypowervm.tasks import scsi_mapper as tsk_map
@@ -33,33 +35,36 @@ from pypowervm.wrappers import storage as pvm_stg
 from pypowervm.wrappers import virtual_io_server as pvm_vios
 
 from nova_powervm import conf as cfg
-from nova_powervm.virt.powervm.i18n import _LI
 from nova_powervm.virt.powervm import vm
 
+
 LOG = logging.getLogger(__name__)
 
 CONF = cfg.CONF
 
 _LLA_SUBNET = "fe80::/64"
 
+CFG_DRV_PREFIX = "cfg_"
+CFG_DRV_SUFFIX = ".iso"
+
 
 class ConfigDrivePowerVM(object):
 
-    def __init__(self, adapter, host_uuid):
+    def __init__(self, adapter):
         """Creates the config drive manager for PowerVM.
 
         :param adapter: The pypowervm adapter to communicate with the system.
-        :param host_uuid: The UUID of the host system.
         """
         self.adapter = adapter
-        self.host_uuid = host_uuid
 
         # Validate that the virtual optical exists
         self.vios_uuid, self.vg_uuid = tsk_vopt.validate_vopt_repo_exists(
-            self.adapter, CONF.powervm.vopt_media_volume_group,
-            CONF.powervm.vopt_media_rep_size)
+            self.adapter,
+            vopt_media_volume_group=CONF.powervm.vopt_media_volume_group,
+            vopt_media_rep_size=CONF.powervm.vopt_media_rep_size)
 
-    def _sanitize_network_info(self, network_info):
+    @staticmethod
+    def _sanitize_network_info(network_info):
         """Will sanitize the network info for the config drive.
 
         Newer versions of cloud-init look at the vif type information in
@@ -91,7 +96,7 @@ class ConfigDrivePowerVM(object):
         :return iso_path: The path to the ISO
         :return file_name: The file name for the ISO
         """
-        LOG.info(_LI("Creating config drive for instance: %s"), instance.name)
+        LOG.info("Creating config drive.", instance=instance)
         extra_md = {}
         if admin_pass is not None:
             extra_md['admin_pass'] = admin_pass
@@ -104,21 +109,41 @@ class ConfigDrivePowerVM(object):
                                                      extra_md=extra_md,
                                                      network_info=network_info)
 
+        # fix instance uuid to match uuid assigned to VM
+        inst_md.uuid = vm.get_pvm_uuid(instance).lower()
+
         # Make sure the path exists.
         im_path = CONF.powervm.image_meta_local_path
         if not os.path.exists(im_path):
             os.mkdir(im_path)
 
         file_name = pvm_util.sanitize_file_name_for_api(
-            instance.name, prefix='cfg_', suffix='.iso',
+            instance.name, prefix=CFG_DRV_PREFIX, suffix=CFG_DRV_SUFFIX,
             max_len=pvm_const.MaxLen.VOPT_NAME)
         iso_path = os.path.join(im_path, file_name)
         with configdrive.ConfigDriveBuilder(instance_md=inst_md) as cdb:
-            LOG.info(_LI("Config drive ISO being built for instance %(inst)s "
-                         "building to path %(iso_path)s."),
-                     {'inst': instance.name, 'iso_path': iso_path})
-            cdb.make_drive(iso_path)
-            return iso_path, file_name
+            LOG.info("Config drive ISO building to path %(iso_path)s.",
+                     {'iso_path': iso_path}, instance=instance)
+            # In case, if there's an OSError related failure while
+            # creating config drive, retry make drive operation.
+
+            def _retry_on_oserror(exc):
+                return isinstance(exc, OSError)
+
+            @retrying.retry(retry_on_exception=_retry_on_oserror,
+                            stop_max_attempt_number=2)
+            def _make_cfg_drive(iso_path):
+                cdb.make_drive(iso_path)
+            try:
+                _make_cfg_drive(iso_path)
+                return iso_path, file_name
+            except OSError:
+                with excutils.save_and_reraise_exception(logger=LOG):
+                    # If we get here, that means there's an exception during
+                    # second attempt, log the same and fail the deploy
+                    # operation.
+                    LOG.exception("Config drive ISO could not be built.",
+                                  instance=instance)
 
     def create_cfg_drv_vopt(self, instance, injected_files, network_info,
                             lpar_uuid, admin_pass=None, mgmt_cna=None,
@@ -184,10 +209,9 @@ class ConfigDrivePowerVM(object):
 
         # Define the function to build and add the mapping
         def add_func(vios_w):
-            LOG.info(_LI("Adding cfg drive mapping for instance %(inst)s for "
-                         "Virtual I/O Server %(vios)s"),
-                     {'inst': instance.name, 'vios': vios_w.name})
-            mapping = tsk_map.build_vscsi_mapping(self.host_uuid, vios_w,
+            LOG.info("Adding config drive mapping to Virtual I/O Server "
+                     "%(vios)s", {'vios': vios_w.name}, instance=instance)
+            mapping = tsk_map.build_vscsi_mapping(None, vios_w,
                                                   lpar_uuid, vopt)
             return tsk_map.add_map(vios_w, mapping)
 
@@ -256,10 +280,8 @@ class ConfigDrivePowerVM(object):
         # immediately
         if stg_ftsk is None:
             built_stg_ftsk = True
-            vio_resp = self.adapter.read(
-                pvm_vios.VIOS.schema_type, root_id=self.vios_uuid,
-                xag=[pvm_const.XAG.VIO_SMAP])
-            vio_w = pvm_vios.VIOS.wrap(vio_resp)
+            vio_w = pvm_vios.VIOS.get(self.adapter, root_id=self.vios_uuid,
+                                      xag=[pvm_const.XAG.VIO_SMAP])
             stg_ftsk = pvm_tx.FeedTask('media_detach', [vio_w])
         else:
             built_stg_ftsk = False
@@ -315,13 +337,12 @@ class ConfigDrivePowerVM(object):
         media_elems = [x.backing_storage for x in media_mappings]
 
         def rm_vopt():
-            LOG.info(_LI("Removing virtual optical for VM with UUID %s."),
+            LOG.info("Removing virtual optical for VM with UUID %s.",
                      lpar_uuid)
-            vg_rsp = self.adapter.read(pvm_vios.VIOS.schema_type,
-                                       root_id=self.vios_uuid,
-                                       child_type=pvm_stg.VG.schema_type,
-                                       child_id=self.vg_uuid)
-            tsk_stg.rm_vg_storage(pvm_stg.VG.wrap(vg_rsp), vopts=media_elems)
+            vg_wrap = pvm_stg.VG.get(self.adapter, uuid=self.vg_uuid,
+                                     parent_type=pvm_vios.VIOS,
+                                     parent_uuid=self.vios_uuid)
+            tsk_stg.rm_vg_storage(vg_wrap, vopts=media_elems)
 
         # Don't add this task if there is no media to delete (eg. config drive)
         if media_elems:
