@@ -17,10 +17,12 @@
 from oslo_concurrency import lockutils
 from oslo_log import log as logging
 from oslo_serialization import jsonutils
+from oslo_utils import uuidutils
 import re
 import six
 
 from nova.compute import power_state
+from nova.compute import task_states
 from nova import exception
 from nova import objects
 from nova.virt import event
@@ -231,6 +233,7 @@ class VMBuilder(object):
     _PVM_DED_SHAR_MODE = 'powervm:dedicated_sharing_mode'
     _PVM_SHAR_PROC_POOL = 'powervm:shared_proc_pool_name'
     _PVM_SRR_CAPABILITY = 'powervm:srr_capability'
+    _PVM_PPT_RATIO = 'powervm:ppt_ratio'
 
     # Map of PowerVM extra specs to the lpar builder attributes.
     # '' is used for attributes that are not implemented yet.
@@ -247,6 +250,8 @@ class VMBuilder(object):
         'powervm:dedicated_proc': lpar_bldr.DED_PROCS,
         'powervm:shared_weight': lpar_bldr.UNCAPPED_WEIGHT,
         'powervm:availability_priority': lpar_bldr.AVAIL_PRIORITY,
+        'powervm:enable_lpar_metric': lpar_bldr.ENABLE_LPAR_METRIC,
+        _PVM_PPT_RATIO: lpar_bldr.PPT_RATIO,
         _PVM_UNCAPPED: None,
         _PVM_DED_SHAR_MODE: None,
         _PVM_PROC_COMPAT: None,
@@ -284,6 +289,8 @@ class VMBuilder(object):
             kwargs['avail_priority'] = cur_lpar_w.avail_priority
             kwargs['srr'] = cur_lpar_w.srr_enabled
             kwargs['proc_compat'] = cur_lpar_w.proc_compat_mode
+            kwargs['enable_lpar_metric'] = (
+                cur_lpar_w.allow_perf_data_collection)
         if slot_mgr is not None:
             # This will already default if not set
             max_vslots = slot_mgr.build_map.get_max_vslots()
@@ -337,6 +344,21 @@ class VMBuilder(object):
             # derive the complex type
             if bldr_key is None:
                 self._build_complex_type(key, attrs, instance.flavor)
+            elif bldr_key == lpar_bldr.ENABLE_LPAR_METRIC:
+                lpar_metric = self._flavor_bool(
+                    instance.flavor.extra_specs[key], key)
+                attrs[bldr_key] = lpar_metric
+            elif bldr_key == lpar_bldr.PPT_RATIO:
+                if (instance.task_state == task_states.REBUILD_SPAWNING and not
+                   self.host_w.get_capability(
+                       'physical_page_table_ratio_capable')):
+                    # We still want to be able to rebuild from hosts that
+                    # support setting the PPT ratio to hosts that don't support
+                    # setting the PPT ratio.
+                    LOG.info("Ignoring PPT ratio on rebuild to PPT ratio "
+                             "unsupported host.", instance=instance)
+                else:
+                    attrs[bldr_key] = instance.flavor.extra_specs[key]
             else:
                 # We found a direct mapping
                 attrs[bldr_key] = instance.flavor.extra_specs[key]
@@ -470,7 +492,7 @@ def get_instance_wrapper(adapter, instance, xag=None):
     """Get the LPAR wrapper for a given Nova instance.
 
     :param adapter: The adapter for the pypowervm API
-    :param instance: The nova instance.
+    :param instance: The nova instance OR its instance uuid.
     :param xag: The pypowervm XAG to be used on the read request
     :return: The pypowervm logical_partition wrapper.
     """
@@ -541,6 +563,19 @@ def get_vm_qp(adapter, lpar_uuid, qprop=None, log_errors=True):
     return jsonutils.loads(resp.body)
 
 
+def get_vm_info(adapter, instance):
+    """Get the InstanceInfo for an instance.
+
+    :param adapter: The pypowervm.adapter.Adapter for the PowerVM REST API.
+    :param instance: nova.objects.instance.Instance object
+    :returns: An InstanceInfo object.
+    """
+    pvm_uuid = get_pvm_uuid(instance)
+    pvm_state = get_vm_qp(adapter, pvm_uuid, 'PartitionState')
+    nova_state = _translate_vm_state(pvm_state)
+    return hardware.InstanceInfo(nova_state)
+
+
 def create_lpar(adapter, host_wrapper, instance, nvram=None, slot_mgr=None):
     """Create an LPAR based on the host based on the instance
 
@@ -751,10 +786,17 @@ def get_pvm_uuid(instance):
     algorithm against the instance's uuid to convert it to the PowerVM
     UUID.
 
-    :param instance: nova.objects.instance.Instance
+    :param instance: nova.objects.instance.Instance OR the OpenStack instance
+                     uuid.
     :return: pvm_uuid.
     """
-    return pvm_uuid.convert_uuid_to_pvm(instance.uuid).upper()
+    # NOTE(esberglu): To work around bug ##1766692, we explicitly use str()
+    # rather than six.text_type here because of this pypowervm check for
+    # isinstance(..., str) at L50 of
+    # https://github.com/powervm/pypowervm/blob/1.1.10/pypowervm/utils/uuid.py
+    inst_uuid = (instance if uuidutils.is_uuid_like(instance)
+                 else str(instance.uuid))
+    return pvm_uuid.convert_uuid_to_pvm(inst_uuid).upper()
 
 
 def _uuid_set_high_bit(pvm_uuid):
